@@ -2,11 +2,11 @@ use crate::command_prelude::*;
 
 use cargo::ops;
 
-pub fn cli() -> App {
+pub fn cli() -> Command {
     subcommand("new")
         .about("Create a new cargo package at <path>")
         .arg_quiet()
-        .arg(Arg::new("path").required(true))
+        .arg(Arg::new("path").action(ArgAction::Set).required(true))
         .arg(opt("registry", "Registry to use").value_name("REGISTRY"))
         .arg_new_opts()
         .after_help("Run `cargo help new` for more detailed information.\n")
