@@ -1,17 +1,17 @@
 //! Core of cargo-add command
 
 mod crate_spec;
-mod dependency;
-mod manifest;
 
 use std::collections::BTreeMap;
 use std::collections::BTreeSet;
 use std::collections::VecDeque;
+use std::fmt::Write;
 use std::path::Path;
 
 use anyhow::Context as _;
 use cargo_util::paths;
 use indexmap::IndexSet;
+use itertools::Itertools;
 use termcolor::Color::Green;
 use termcolor::Color::Red;
 use termcolor::ColorSpec;
@@ -26,18 +26,17 @@ use crate::core::Registry;
 use crate::core::Shell;
 use crate::core::Summary;
 use crate::core::Workspace;
+use crate::util::toml_mut::dependency::Dependency;
+use crate::util::toml_mut::dependency::GitSource;
+use crate::util::toml_mut::dependency::MaybeWorkspace;
+use crate::util::toml_mut::dependency::PathSource;
+use crate::util::toml_mut::dependency::Source;
+use crate::util::toml_mut::dependency::WorkspaceSource;
+use crate::util::toml_mut::manifest::DepTable;
+use crate::util::toml_mut::manifest::LocalManifest;
 use crate::CargoResult;
 use crate::Config;
 use crate_spec::CrateSpec;
-use dependency::Dependency;
-use dependency::GitSource;
-use dependency::PathSource;
-use dependency::RegistrySource;
-use dependency::Source;
-use manifest::LocalManifest;
-
-use crate::ops::cargo_add::dependency::{MaybeWorkspace, WorkspaceSource};
-pub use manifest::DepTable;
 
 /// Information on what dependencies should be added
 #[derive(Clone, Debug)]
@@ -102,7 +101,7 @@ pub fn add(workspace: &Workspace<'_>, options: &AddOptions<'_>) -> CargoResult<(
             table_option.map_or(true, |table| is_sorted(table.iter().map(|(name, _)| name)))
         });
     for dep in deps {
-        print_msg(&mut options.config.shell(), &dep, &dep_table)?;
+        print_action_msg(&mut options.config.shell(), &dep, &dep_table)?;
         if let Some(Source::Path(src)) = dep.source() {
             if src.path == manifest.path.parent().unwrap_or_else(|| Path::new("")) {
                 anyhow::bail!(
@@ -127,11 +126,63 @@ pub fn add(workspace: &Workspace<'_>, options: &AddOptions<'_>) -> CargoResult<(
                 inherited_features.iter().map(|s| s.as_str()).collect();
             unknown_features.extend(inherited_features.difference(&available_features).copied());
         }
+
         unknown_features.sort();
+
         if !unknown_features.is_empty() {
-            anyhow::bail!("unrecognized features: {unknown_features:?}");
+            let (mut activated, mut deactivated) = dep.features();
+            // Since the unknown features have been added to the DependencyUI we need to remove
+            // them to present the "correct" features that can be specified for the crate.
+            deactivated.retain(|f| !unknown_features.contains(f));
+            activated.retain(|f| !unknown_features.contains(f));
+
+            let mut message = format!(
+                "unrecognized feature{} for crate {}: {}\n",
+                if unknown_features.len() == 1 { "" } else { "s" },
+                dep.name,
+                unknown_features.iter().format(", "),
+            );
+            if activated.is_empty() && deactivated.is_empty() {
+                write!(message, "no features available for crate {}", dep.name)?;
+            } else {
+                if !deactivated.is_empty() {
+                    writeln!(
+                        message,
+                        "disabled features:\n    {}",
+                        deactivated
+                            .iter()
+                            .map(|s| s.to_string())
+                            .coalesce(|x, y| if x.len() + y.len() < 78 {
+                                Ok(format!("{x}, {y}"))
+                            } else {
+                                Err((x, y))
+                            })
+                            .into_iter()
+                            .format("\n    ")
+                    )?
+                }
+                if !activated.is_empty() {
+                    writeln!(
+                        message,
+                        "enabled features:\n    {}",
+                        activated
+                            .iter()
+                            .map(|s| s.to_string())
+                            .coalesce(|x, y| if x.len() + y.len() < 78 {
+                                Ok(format!("{x}, {y}"))
+                            } else {
+                                Err((x, y))
+                            })
+                            .into_iter()
+                            .format("\n    ")
+                    )?
+                }
+            }
+            anyhow::bail!(message.trim().to_owned());
         }
 
+        print_dep_table_msg(&mut options.config.shell(), &dep)?;
+
         manifest.insert_into_table(&dep_table, &dep)?;
         manifest.gc_dep(dep.toml_key());
     }
@@ -637,6 +688,42 @@ impl DependencyUI {
             })
             .collect();
     }
+
+    fn features(&self) -> (IndexSet<&str>, IndexSet<&str>) {
+        let mut activated: IndexSet<_> =
+            self.features.iter().flatten().map(|s| s.as_str()).collect();
+        if self.default_features().unwrap_or(true) {
+            activated.insert("default");
+        }
+        activated.extend(self.inherited_features.iter().flatten().map(|s| s.as_str()));
+        let mut walk: VecDeque<_> = activated.iter().cloned().collect();
+        while let Some(next) = walk.pop_front() {
+            walk.extend(
+                self.available_features
+                    .get(next)
+                    .into_iter()
+                    .flatten()
+                    .map(|s| s.as_str()),
+            );
+            activated.extend(
+                self.available_features
+                    .get(next)
+                    .into_iter()
+                    .flatten()
+                    .map(|s| s.as_str()),
+            );
+        }
+        activated.remove("default");
+        activated.sort();
+        let mut deactivated = self
+            .available_features
+            .keys()
+            .filter(|f| !activated.contains(f.as_str()) && *f != "default")
+            .map(|f| f.as_str())
+            .collect::<IndexSet<_>>();
+        deactivated.sort();
+        (activated, deactivated)
+    }
 }
 
 impl<'s> From<&'s Summary> for DependencyUI {
@@ -700,9 +787,7 @@ fn populate_available_features(
     Ok(dependency)
 }
 
-fn print_msg(shell: &mut Shell, dep: &DependencyUI, section: &[String]) -> CargoResult<()> {
-    use std::fmt::Write;
-
+fn print_action_msg(shell: &mut Shell, dep: &DependencyUI, section: &[String]) -> CargoResult<()> {
     if matches!(shell.verbosity(), crate::core::shell::Verbosity::Quiet) {
         return Ok(());
     }
@@ -739,38 +824,14 @@ fn print_msg(shell: &mut Shell, dep: &DependencyUI, section: &[String]) -> Cargo
     };
     write!(message, " {section}")?;
     write!(message, ".")?;
-    shell.status("Adding", message)?;
+    shell.status("Adding", message)
+}
 
-    let mut activated: IndexSet<_> = dep.features.iter().flatten().map(|s| s.as_str()).collect();
-    if dep.default_features().unwrap_or(true) {
-        activated.insert("default");
+fn print_dep_table_msg(shell: &mut Shell, dep: &DependencyUI) -> CargoResult<()> {
+    if matches!(shell.verbosity(), crate::core::shell::Verbosity::Quiet) {
+        return Ok(());
     }
-    activated.extend(dep.inherited_features.iter().flatten().map(|s| s.as_str()));
-    let mut walk: VecDeque<_> = activated.iter().cloned().collect();
-    while let Some(next) = walk.pop_front() {
-        walk.extend(
-            dep.available_features
-                .get(next)
-                .into_iter()
-                .flatten()
-                .map(|s| s.as_str()),
-        );
-        activated.extend(
-            dep.available_features
-                .get(next)
-                .into_iter()
-                .flatten()
-                .map(|s| s.as_str()),
-        );
-    }
-    activated.remove("default");
-    activated.sort();
-    let mut deactivated = dep
-        .available_features
-        .keys()
-        .filter(|f| !activated.contains(f.as_str()) && *f != "default")
-        .collect::<Vec<_>>();
-    deactivated.sort();
+    let (activated, deactivated) = dep.features();
     if !activated.is_empty() || !deactivated.is_empty() {
         let prefix = format!("{:>13}", " ");
         let suffix = if let Some(version) = &dep.available_version {
