@@ -1,6 +1,6 @@
 //! Tests for some invalid .cargo/config files.
 
-use cargo_test_support::registry::Package;
+use cargo_test_support::registry::{self, Package};
 use cargo_test_support::{basic_manifest, project, rustc_host};
 
 #[cargo_test]
@@ -19,8 +19,8 @@ fn bad1() {
         .with_status(101)
         .with_stderr(
             "\
-[ERROR] invalid configuration for key `target.nonexistent-target`
-expected a table, but found a string for `[..]` in [..]config
+[ERROR] expected table for configuration key `target.nonexistent-target`, \
+but found string in [..]config
 ",
         )
         .run();
@@ -62,6 +62,7 @@ Caused by:
 
 #[cargo_test]
 fn bad3() {
+    let registry = registry::init();
     let p = project()
         .file("src/lib.rs", "")
         .file(
@@ -75,6 +76,7 @@ fn bad3() {
     Package::new("foo", "1.0.0").publish();
 
     p.cargo("publish -v")
+        .replace_crates_io(registry.index_url())
         .with_status(101)
         .with_stderr(
             "\
@@ -113,6 +115,7 @@ Caused by:
 
 #[cargo_test]
 fn bad6() {
+    let registry = registry::init();
     let p = project()
         .file("src/lib.rs", "")
         .file(
@@ -126,6 +129,7 @@ fn bad6() {
     Package::new("foo", "1.0.0").publish();
 
     p.cargo("publish -v")
+        .replace_crates_io(registry.index_url())
         .with_status(101)
         .with_stderr(
             "\
@@ -209,7 +213,7 @@ fn duplicate_packages_in_cargo_lock() {
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.0.1"
                 authors = []
@@ -263,7 +267,7 @@ fn bad_source_in_cargo_lock() {
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
                 name = "foo"
                 version = "0.0.1"
                 authors = []
@@ -654,7 +658,7 @@ warning: unused manifest key: target.foo.bar
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
 
                 name = "foo"
                 version = "0.5.0"
@@ -667,7 +671,7 @@ warning: unused manifest key: target.foo.bar
     p.cargo("build")
         .with_stderr(
             "\
-warning: unused manifest key: project.bulid
+warning: unused manifest key: package.bulid
 [COMPILING] foo [..]
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
@@ -679,7 +683,7 @@ warning: unused manifest key: project.bulid
         .file(
             "Cargo.toml",
             r#"
-                [project]
+                [package]
 
                 name = "foo"
                 version = "0.5.0"
